/**
 *  Copyright Notice:
 *  Copyright 2021-2022 DMTF. All rights reserved.
 *  License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/libspdm/blob/main/LICENSE.md
 **/

#include "spdm_requester.h"

void libspdm_dump_hex_str(const uint8_t *buffer, size_t buffer_size)
{
}

bool libspdm_read_input_file(const char *file_name, void **file_data,
                             size_t *file_size)
{
    return true;
}

bool libspdm_write_output_file(const char *file_name, const void *file_data,
                               size_t file_size)
{
    return true;
}
