/**
 *  Copyright Notice:
 *  Copyright 2021-2022 DMTF. All rights reserved.
 *  License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/libspdm/blob/main/LICENSE.md
 **/

#include "hal/base.h"

void *allocate_pool(size_t AllocationSize)
{
    return NULL;
}

void *allocate_zero_pool(size_t AllocationSize)
{
    return NULL;
}

void free_pool(const void *buffer)
{
}
