/** @file
  Wrapper header file to include <Register/Intel/Cpuid.h> in MdePkg.

  Copyright (c) 2015 - 2019, Intel Corporation. All rights reserved.<BR>
  SPDX-License-Identifier: BSD-2-Clause-Patent
**/

#pragma once

#include <Register/Intel/Cpuid.h>
#include <Register/Amd/Cpuid.h>
