/** @file
  Include file to support building the third-party brotli.

  Copyright (C) 2024 Advanced Micro Devices, Inc. All rights reserved.<BR>
  SPDX-License-Identifier: BSD-2-Clause-Patent

**/

#include <BrotliDecUefiSupport.h>
