/** @file
  Internal include file for TlsLibNull.

Copyright (c) 2016 - 2017, Intel Corporation. All rights reserved.<BR>
SPDX-License-Identifier: BSD-2-Clause-Patent

**/

#pragma once

#include <Library/BaseCryptLib.h>
#include <Library/DebugLib.h>
#include <Library/BaseLib.h>
