/** @file
  ARM IORT parser common functionality.

  Copyright (c) 2025, Arm Limited. All rights reserved.<BR>
  SPDX-License-Identifier: BSD-2-Clause-Patent
**/

#pragma once

UINT16
EFIAPI
GetNextIortIdentifier (
  );
